 // Automatic Xylophone
 // Details on:
 // https://homofaciens.de/technics-physical-computing-examples-Xylophone_en.htm
 
 $fn = 100;

M3_hole = 3.5;
M3_nut_e = 6.3;
M4_hole = 4.5;
M5_hole = 5.5;
M6_hole = 6.5;
M4_nut_e = 7.66 + 0.3;
M5_nut_e = 8.79 + 0.3;
M6_nut_e = 11.05 + 0.3;
M8_nut_e = 14.38+1.5;
 


inductor();
//translate([17, 0, 0]) beater();

//multiple_beaters();

//spacer_bar();

//guide();

module guide(){
  g_hole = M3_hole;
  g_h = 10;
  g_socket = 5;
  g_d1 = 5;
  g_wall = 1;
  g_d2 = g_hole + 2* g_wall;
  g_d3 = 20;
  
  translate([0, 0, g_h/2]){
    difference(){
      cylinder(d=g_d3, h=g_h, center=true);
      translate([0, 0, g_socket]){
        difference(){
          union(){
            cylinder(d=16+0.5, h=g_h, center=true);
            cube(size=[12.5, 100, g_h], center=true);
          }
          cylinder(d=g_d2, h=g_h*1.1, center=true);
        }
      }
      cylinder(d=g_hole, h=g_h*1.1, center=true);
    }
  }
}

module spacer_bar(){
  sb_w = 16;
  sb_wall = 2;
  sb_h = 7;
  sb_dis = 25;
  sb_no = 7;
  sb_l = sb_no * sb_dis;
  
  translate([0, 0, sb_wall/2])
  cube(size=[sb_l, sb_w, sb_wall], center=true);
  for(i=[0:1:sb_no-1]){
    translate([(i - sb_no/2) * sb_dis + sb_w/2, 0, sb_h/2]){
      if(i!=2 && i!=5)
      cylinder(d=sb_w, h=sb_h, center=true);
    }
  }
  
  
}

module multiple_beaters(){
  for(i=[0:1:4]){
    for(j=[0:1:4]){
      translate([10*i, 10*j, 0]){
        beater();
      }
    }
  }

  difference(){
    translate([20, 20, 0])
    roundedRect([60, 60, 0.35], 10);
    translate([20, 20, 0.35/2])
    cube(size=[55-5, 55-5, 1.4], center=true);
  }
}


module beater(){
  b_d = 7;
  b_h = 12;
  brim = 0.35;
  
  intersection(){
    union(){
      translate([0, 0, (b_h-b_d/2)/2])
      cylinder(d=b_d, h=b_h-b_d/2, center=true);
      translate([0, 0, b_h-b_d/2]){
        sphere(d=b_d);
      }
    }
    translate([0, 0, b_h]){
      cylinder(d=b_d*2, h=b_h*2, center=true);
    }
  }
  translate([0, 0, brim/2]){
    rotate([0, 0, 45])
    cube(size=[b_d + 8, 1.4, brim], center=true);
    rotate([0, 0, -45])
    cube(size=[b_d + 8, 1.4, brim], center=true);
  }
    
}

module inductor(){
  i_di = 8.5;
  i_wall = 1;
  i_ring_1 = 8;
  i_ring_2 = 3;
  i_do = 16.0;
  i_h = 21;
  i_hole = 1.5;
  
  translate([0, 0, i_h/2]){
    difference(){
      union(){
        cylinder(d=i_di+2*i_wall, h=i_h, center=true);
        translate([0, 0, (i_ring_1-i_h)/2]){
          cylinder(d=i_do, h=i_ring_1, center=true);
        }
      }
      translate([0, 0, 4])
      cylinder(d=i_di, h=i_h*1., center=true);
      cylinder(d=M4_nut_e, h=i_h*1.1, center=true, $fn=6);
    }
  }
  translate([0, 0, i_h/2]){
    translate([i_do + 1, 0, (i_ring_2-i_h)/2]){
      difference(){
        cylinder(d=i_do, h=i_ring_2, center=true);
        cylinder(d=i_di+2*i_wall+0.17, h=i_h*1.2, center=true);
        translate([1, 0, 0])
        cube(size=[i_di+3, i_hole, i_h], center=true);
        translate([i_do/2, 0, 1])
        cube(size=[i_do, i_hole, i_ring_2], center=true);
      }
    }
  }
}

module roundedRect(size, radius, edges=[1, 1, 1, 1], vscale=[1,1], shift=0){
  x = size[0];
  y = size[1];
  z = size[2];

  translate([0, -shift*2, 0]){
    linear_extrude(height=z, scale=[vscale[0], vscale[1]]){
      translate([0, shift*2, 0]){
        hull(){
            // place 4 circles in the corners, with the given radius
            if(edges[0]){
              translate([(-x/2)+(radius), (-y/2)+(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(-x/2)+(radius/2), (-y/2)+(radius/2), 0])
              square(size=[radius, radius], center=true);
            }
            if(edges[1]){
              translate([(x/2)-(radius), (-y/2)+(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(x/2)-(radius/2), (-y/2)+(radius/2), 0])
              square(size=[radius, radius], center=true);
            }

            if(edges[2]){
              translate([(-x/2)+(radius), (y/2)-(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(-x/2)+(radius/2), (y/2)-(radius/2), 0])
              square(size=[radius, radius], center=true);
            }

            if(edges[3]){
              translate([(x/2)-(radius), (y/2)-(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(x/2)-(radius/2), (y/2)-(radius/2), 0])
              square(size=[radius, radius], center=true);
            }
          }
       }
    }
  }
}